<?php
namespace Tripetto;

class Loader
{
    static function data($reference, $token)
    {
        if (Helpers::isValidSHA256($reference)) {
            global $wpdb;

            $form = $wpdb->get_row($wpdb->prepare("SELECT * from {$wpdb->prefix}tripetto_forms where reference=%s", $reference));

            if (!is_null($form) && Helpers::isValidJSON($form->definition)) {
                $response = new \stdClass();
                $runner = "autoscroll";

                if (!empty($form->runner)) {
                    $runner = $form->runner;
                } elseif (!empty($form->collector)) {
                    switch ($form->collector) {
                        case "standard-bootstrap":
                            $runner = "classic";
                            break;
                        default:
                            $runner = "autoscroll";
                            break;
                    }
                }

                $response->definition = Migration::definition($form, json_decode($form->definition, false));
                $response->styles = Helpers::isValidJSON($form->styles)
                    ? json_decode($form->styles, false)
                    : Migration::styles($form, $runner);
                $response->l10n = Helpers::isValidJSON($form->l10n) ? json_decode($form->l10n, false) : Migration::l10n($form);

                if (Helpers::isValidSHA256($token)) {
                    $snapshot = $wpdb->get_row(
                        $wpdb->prepare(
                            "SELECT snapshot from {$wpdb->prefix}tripetto_snapshots where form_id=%d AND reference=%s",
                            $form->id,
                            $token
                        )
                    );

                    if (!is_null($snapshot) && !empty($snapshot->snapshot)) {
                        $response->snapshot = json_decode($snapshot->snapshot, false);
                    }
                }

                header("Content-Type: application/json");

                echo str_replace('"_empty_":', '"":', json_encode($response));
                http_response_code(200);
            } else {
                http_response_code(404);
            }
        } else {
            http_response_code(400);
        }

        die();
    }

    static function load()
    {
        Loader::data(!empty($_POST["reference"]) ? $_POST["reference"] : "", !empty($_POST["token"]) ? $_POST["token"] : "");
    }

    static function process()
    {
        define("SHORTINIT", true);

        if (file_exists(stream_resolve_include_path($_SERVER["DOCUMENT_ROOT"] . "/wp-load.php"))) {
            require_once $_SERVER["DOCUMENT_ROOT"] . "/wp-load.php";
        } else {
            $root = dirname(dirname(dirname(dirname(__DIR__))));

            if (file_exists(stream_resolve_include_path($root . "/wp-load.php"))) {
                require_once $root . "/wp-load.php";
            } else {
                http_response_code(303);

                die();

                return;
            }
        }

        require_once __DIR__ . "/helpers.php";
        require_once __DIR__ . "/migration.php";

        header("Cache-Control: no-store, max-age=0");
        header("Pragma: no-cache");

        Loader::data(!empty($_GET["reference"]) ? $_GET["reference"] : "", !empty($_GET["token"]) ? $_GET["token"] : "");
    }

    static function register($plugin)
    {
        add_action("wp_ajax_tripetto_load", ["Tripetto\Loader", "load"]);
        add_action("wp_ajax_nopriv_tripetto_load", ["Tripetto\Loader", "load"]);
    }
}
?>
